@@ -68,7 +68,7 @@ cb_show_edit_menu(GtkAction *action,
 
 	dv = FB_DIRVIEW (fb->dirview);
 
-	flag = (dv->writable && (paste_path != NULL));
+	flag = (dv->writable && (paste_list != NULL));
 	ui_set_sensitive(fb->ui_manager, "/MenuBar/EditMenu/Paste", flag);
 
 	flag = fb_dirview_row_get (dv,
@@ -252,119 +252,114 @@ static void
 cb_copy(GtkAction *action,
 	gpointer data)
 {
+	gchar *path;
 	FbWindow *fb = data;
 
-	if (paste_path) g_free(paste_path);
- 
+	if (paste_list) {
+		g_list_foreach (paste_list, (GFunc) g_free, NULL);
+		g_list_free (paste_list);
+		paste_list = NULL;
+	}
+
 	fb_dirview_row_get (FB_DIRVIEW (fb->dirview),
-			    COLUMN_PATH, &paste_path,
+			    COLUMN_PATH, &path,
 			    -1);
+	paste_list = g_list_append (paste_list, path);
 	is_copy = TRUE;
 
-	ui_set_message(fb->ui_manager, "/MenuBar/EditMenu/Paste", paste_path);
-	ui_set_message(fb->ui_manager, "/DirViewPopup/Paste", paste_path);
+	ui_set_message(fb->ui_manager, "/MenuBar/EditMenu/Paste", path);
+	ui_set_message(fb->ui_manager, "/DirViewPopup/Paste", path);
 }
 
 static void
 cb_cut(GtkAction *action,
        gpointer data)
 {
+	gchar *path;
 	FbWindow *fb = data;
 
-	if (paste_path) g_free(paste_path);
- 
+	if (paste_list) {
+		g_list_foreach (paste_list, (GFunc) g_free, NULL);
+		g_list_free (paste_list);
+		paste_list = NULL;
+	}
+
 	fb_dirview_row_get (FB_DIRVIEW (fb->dirview),
-			    COLUMN_PATH, &paste_path,
+			    COLUMN_PATH, &path,
 			    -1);
+	paste_list = g_list_append (paste_list, path);
 	is_copy = FALSE;
 
-	ui_set_message(fb->ui_manager, "/MenuBar/EditMenu/Paste", paste_path);
-	ui_set_message(fb->ui_manager, "/DirViewPopup/Paste", paste_path);
+	ui_set_message(fb->ui_manager, "/MenuBar/EditMenu/Paste", path);
+	ui_set_message(fb->ui_manager, "/DirViewPopup/Paste", path);
 }
 
 static void
 cb_paste(GtkAction *action,
 	 gpointer data)
 {
-	gboolean result = FALSE;
-	gchar *tmp, *dest_path;
+	gboolean result;
+	gint step, len;
+	gchar *tmp, *src, *dest;
 	const gchar *workdir;
+	GList *pathl;
 	FbWindow *fb = data;
 
-	if (! paste_path) return;
+	if (! paste_list) return;
 
 	fb_statusbar_pop ();
 
 	workdir = fb_dirview_get_workdir (FB_DIRVIEW (fb->dirview));
 
-/* 	fb_statusbar_progress_start (FB_STATUSBAR (fb->status_bar)); */
-	if (path_is_dir (paste_path)) {
-		gint len, step;
-		GList *node, *path_list;
+	/* 	fb_statusbar_progress_start (FB_STATUSBAR (fb->status_bar)); */
+	step = 1;
+	len = g_list_length (paste_list);
+	for (pathl = paste_list; pathl; pathl = pathl->next) {
+		while (gtk_events_pending ()) gtk_main_iteration ();
 
-		path_list = get_path_list (paste_path, TRUE);
-		step = 1;
-		len = g_list_length (path_list);
-		for (node = path_list; node; node = node->next) {
-			while ( gtk_events_pending ()) gtk_main_iteration ();
+		src = pathl->data;
+		result  = FALSE;
 
-			tmp = g_path_get_dirname (paste_path);
-			dest_path = replace_string (node->data, tmp, workdir);
-			g_free (tmp);
-
-			if (path_is_dir (node->data)) {
-				result = ensure_dir_exists (dest_path, 0755);
-				if (! is_copy && result)
-					result = ! rmdir (node->data);
-			}
-			else {
-				result = file_copy (node->data, dest_path);
-				if (! is_copy && result)
-					result = ! unlink (node->data);
-			}
-			g_free (dest_path);
-			fb_statusbar_set_progress (FB_STATUSBAR (fb->status_bar), (gdouble) step / (gdouble) len);
-			step++;
-		}
-		path_list_free (path_list);
-	}
-	else {
-		tmp = g_path_get_basename (paste_path);
-		dest_path = g_strconcat (workdir, "/", tmp, NULL);
+		tmp = g_path_get_basename (src);
+		dest = g_strconcat (workdir, "/", tmp, NULL);
 		g_free (tmp);
 
-		result = file_copy (paste_path, dest_path);
-		if (! is_copy && result)
-			result = ! unlink (paste_path);
-		
-		g_free (dest_path);
-	}
-/* 	fb_statusbar_progress_end (FB_STATUSBAR (fb->status_bar)); */
-
-	if (result) {
-		tmp = g_path_get_basename (paste_path);
-		dest_path = g_strconcat (workdir, "/", tmp, NULL);
-		g_free (tmp);
-
-		fb_dirview_add_path (FB_DIRVIEW (fb->dirview), dest_path);
-		g_free (dest_path);
-
-		if (! is_copy) {
-			g_free (paste_path);
-			paste_path = NULL;
+		result = FALSE;
+		if (path_is_dir (src)) {
+			result = cpdir_recursive (src, dest);
+			if (! is_copy && result)
+				result = rmdir_recursive (src);
 		}
-	}
-	else {
-		GtkWidget *dlg;
+		else if (path_is_file (src)) {
+			result = file_copy (src, dest);
+			if (! is_copy && result)
+				result = ! unlink (src);
+		}
+		fb_statusbar_set_progress (FB_STATUSBAR (fb->status_bar), (gdouble) step / (gdouble) len);
+		/* 	fb_statusbar_progress_end (FB_STATUSBAR (fb->status_bar)); */
+		step++;
 
-		dlg = gtk_message_dialog_new (GTK_WINDOW (fb), GTK_DIALOG_MODAL,
-					      GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
-					      _("Failed paste"));
-		g_signal_connect_swapped (G_OBJECT (dlg), "response",
-					  G_CALLBACK (gtk_widget_destroy), dlg);
-		gtk_widget_show (dlg);
+		if (result) {
+			fb_dirview_add_path (FB_DIRVIEW (fb->dirview), dest);
+		}
+		else {
+			GtkWidget *dlg;
+
+			dlg = gtk_message_dialog_new (GTK_WINDOW (fb), GTK_DIALOG_MODAL,
+						      GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
+						      _("Failed paste"));
+			g_signal_connect_swapped (G_OBJECT (dlg), "response",
+						  G_CALLBACK (gtk_widget_destroy), dlg);
+			gtk_widget_show (dlg);
+		}
+		g_free (dest);
 	}
 
+	if (! is_copy) {
+		g_list_foreach (paste_list, (GFunc) g_free, NULL);
+		g_list_free (paste_list);
+		paste_list = NULL;
+	}
 	fb_statusbar_set_progress (FB_STATUSBAR (fb->status_bar), 0.0);
 }
 
@@ -429,7 +424,7 @@ cb_delete(GtkAction *action,
 		}
 		path_list_free (path_list);
 	}
-	else {
+	else if (path_is_file (path)) {
 		if (unlink (path)) {
 			dlg = gtk_message_dialog_new (GTK_WINDOW (fb), GTK_DIALOG_MODAL,
 						      GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
