@@ -66,7 +66,8 @@ static gboolean is_readable                (const gchar *path);
 static GtkTreeIter dirlist_append          (GtkListStore *store,
 					    const gchar  *path,
 					    gboolean      dot_filter,
-					    gboolean      other_filter);
+					    gboolean      other_filter,
+					    gboolean set_size);
 static void        current_dir_append      (GtkListStore *store,
 					    const gchar  *path);
 static void        parent_dir_append       (GtkListStore *store,
@@ -419,7 +420,8 @@ static GtkTreeIter
 dirlist_append(GtkListStore *list,
 	       const gchar *path,
 	       gboolean dot_filter,
-	       gboolean other_filter)
+	       gboolean other_filter,
+	       gboolean set_size)
 {
 	struct stat file;
 	gboolean readable;
@@ -461,7 +463,9 @@ dirlist_append(GtkListStore *list,
 	readable = is_readable(path);
 /* 	is_writable (path); */
 
-	if (type == DIRECTORY)
+	if (type == DIRECTORY && set_size)
+		size = str_entries2str(count_dir_entries(path, dot_filter, other_filter));
+	else if (type == DIRECTORY && ! set_size)
 		size = g_strdup ("...");
 	else
 		size = str_size2str(file.st_size);
@@ -506,8 +510,9 @@ current_dir_append (GtkListStore *store,
 {
 	GtkTreeIter iter;
 
-	iter = dirlist_append (store, path, FALSE, FALSE);
+	iter = dirlist_append (store, path, FALSE, FALSE, FALSE);
 	gtk_list_store_set (store, &iter,
+			    COLUMN_TYPE, WORKDIR,
 			    COLUMN_DISPLAY_PATH, ".",
 			    -1);
 }
@@ -525,8 +530,9 @@ parent_dir_append (GtkListStore *store,
 	else {
 		parent_path = remove_level_from_path (path);
 	}
-	iter = dirlist_append (store, parent_path, FALSE, FALSE);
+	iter = dirlist_append (store, parent_path, FALSE, FALSE, FALSE);
 	gtk_list_store_set (store, &iter,
+			    COLUMN_TYPE, PARENTDIR,
 			    COLUMN_DISPLAY_PATH, "..",
 			    -1);
 
@@ -537,17 +543,18 @@ static gint
 dirlist_get_count(GtkListStore *list,
 		  FbFileType type)
 {
-	gint src_type, count = 0;
+	gint cmp, count = 0;
 	GtkTreeIter iter;
 
 	if (type == DIRECTORY) count = 2;
 
 	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(list), &iter);
 	while (gtk_tree_model_iter_next(GTK_TREE_MODEL(list), &iter)) {
+		cmp = -1;
 		gtk_tree_model_get(GTK_TREE_MODEL(list), &iter,
-				   COLUMN_TYPE, &src_type,
+				   COLUMN_TYPE, &cmp,
 				   -1);
-		if (src_type == type) count++;
+		if (cmp == type) count++;
 	}
 
 	return count;
@@ -563,25 +570,22 @@ check_dot(GtkTreeModel *model,
 	  GtkTreeIter *a,
 	  GtkTreeIter *b)
 {
-	gint cmp = 0;
-	gchar *str1, *str2;
+	gint type1, type2;
 
-	gtk_tree_model_get(model, a, COLUMN_DISPLAY_PATH, &str1, -1);
-	gtk_tree_model_get(model, b, COLUMN_DISPLAY_PATH, &str2, -1);
+/* 	type1 = type2 = -1; */
+	gtk_tree_model_get(model, a, COLUMN_TYPE, &type1, -1);
+	gtk_tree_model_get(model, b, COLUMN_TYPE, &type2, -1);
 
-	if (!strcmp(str1, "."))
-		cmp = -1;
-	else if (!strcmp(str2, "."))
-		cmp = 1;
-	else if (!strcmp(str1, ".."))
-		cmp = -1;
-	else if (!strcmp(str2, ".."))
-		cmp = 1;
- 
-	g_free (str1);
-	g_free (str2);
-
-	return cmp;
+	if (type1 == WORKDIR)
+		return -1;
+	else if (type2 == WORKDIR)
+		return 1;
+	else if (type1 == PARENTDIR)
+		return -1;
+	else if (type2 == PARENTDIR)
+		return 1;
+	else
+		return 0;
 }
 
 static gint
@@ -593,8 +597,11 @@ sort_path(GtkTreeModel *model,
 	gint cmp;
 	gchar *str1, *str2;
 
-	gtk_tree_model_get(model, a, COLUMN_DISPLAY_PATH, &str1, -1);
-	gtk_tree_model_get(model, b, COLUMN_DISPLAY_PATH, &str2, -1);
+	cmp = check_dot (model, a, b);
+	if (cmp != 0) return cmp;
+
+	gtk_tree_model_get(model, a, COLUMN_PATH, &str1, -1);
+	gtk_tree_model_get(model, b, COLUMN_PATH, &str2, -1);
 
 	cmp = strcmp(str1, str2);
 
@@ -615,24 +622,12 @@ sort_type(GtkTreeModel *model,
 	gtk_tree_model_get(model, a, COLUMN_TYPE, &type1, -1);
 	gtk_tree_model_get(model, b, COLUMN_TYPE, &type2, -1);
 
-	if ((type1 == DIRECTORY) && (type2 != DIRECTORY))
+	if (type1 < type2)
 		return -1;
-	else if ((type1 != DIRECTORY) && (type2 == DIRECTORY))
+	else if (type2 > type1)
 		return 1;
-	else if ((type1 == ARCHIVE) && (type2 != ARCHIVE))
-		return -1;
-	else if ((type1 != ARCHIVE) && (type2 == ARCHIVE))
-		return 1;
-	else if ((type1 == IMAGE) && (type2 != IMAGE))
-		return -1;
-	else if ((type1 != IMAGE) && (type2 == IMAGE))
-		return 1;
-	else if ((type1 == OTHER) && (type2 != OTHER))
-		return -1;
-	else if ((type1 != OTHER) && (type2 == OTHER))
-		return 1;
-
-	return sort_path(model, a, b, data);
+	else if (type1 == type2)
+		return sort_path (model, a, b, data);
 }
 
 static gint
@@ -641,10 +636,10 @@ sort_size(GtkTreeModel *model,
 	  GtkTreeIter *b,
 	  gpointer data)
 {
-	gint size1, size2, result;
+	gint size1, size2, cmp;
 
-	result = check_dot(model, a, b);
-	if (result) return result;
+	cmp = check_dot(model, a, b);
+	if (cmp) return cmp;
 
 	gtk_tree_model_get(model, a, COLUMN_SIZE, &size1, -1);
 	gtk_tree_model_get(model, b, COLUMN_SIZE, &size2, -1);
@@ -654,7 +649,7 @@ sort_size(GtkTreeModel *model,
 	else if (size1 < size2)
 		return 1;
 	else if (size1 == size2)
-		return 0;
+		return sort_path (model, a, b, data);
 }
 
 static gint
@@ -663,11 +658,11 @@ sort_mtime(GtkTreeModel *model,
 	   GtkTreeIter *b,
 	   gpointer data)
 {
-	gint result;
+	gint cmp;
 	glong time1, time2;
 
-	result = check_dot(model, a, b);
-	if (result) return result;
+	cmp = check_dot(model, a, b);
+	if (cmp) return cmp;
 
 	gtk_tree_model_get(model, a, COLUMN_MTIME, &time1, -1);
 	gtk_tree_model_get(model, b, COLUMN_MTIME, &time2, -1);
@@ -677,7 +672,7 @@ sort_mtime(GtkTreeModel *model,
 	else if (time1 < time2)
 		return -1;
 	else if (time1 == time2)
-		return 0;
+		return sort_path (model, a, b, data);
 }
 
 /*
@@ -785,13 +780,15 @@ cb_name_edited (GtkCellRendererText *cell,
 			    -1);
 	gtk_tree_path_free (tpath);
 
+	src_path = NULL;
 	gtk_tree_model_get(GTK_TREE_MODEL(dv->store), &iter,
 			   COLUMN_PATH, &src_path,
 			   -1);
 	/* FIX ME (add message dialog) */
 	tmp = g_path_get_basename(src_path);
 	if (!strcmp(name, tmp)) {
-		g_free(tmp);
+		g_free (src_path);
+		g_free (tmp);
 		return;
 	}
 	g_free(tmp);
@@ -801,15 +798,17 @@ cb_name_edited (GtkCellRendererText *cell,
 	g_free(tmp);
 
 	if (rename(src_path, new_path)) {
-		g_free(new_path);
+		g_free (src_path);
+		g_free (new_path);
 		return;
 	}
 	/* END */
 
 	gtk_list_store_remove (dv->store, &iter);
-	dirlist_append(dv->store, new_path, dv->dot_filter, dv->other_filter);
+	dirlist_append(dv->store, new_path, dv->dot_filter, dv->other_filter, TRUE);
 
-	g_free(new_path);
+	g_free (src_path);
+	g_free (new_path);
 
 }
 
@@ -842,6 +841,12 @@ fb_dirview_init (FbDirview *dv)
 	gtk_tree_view_enable_model_drag_dest (GTK_TREE_VIEW (dv),
 					      uri_drag_types, n_uri_drag_types,
 					      GDK_ACTION_COPY | GDK_ACTION_MOVE);
+/* 	{ */
+/* 		GtkTreeSelection *selection; */
+
+/* 		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (dv)); */
+/* 		gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE); */
+/* 	} */
 
 	dv->column = gtk_tree_view_column_new();
 	gtk_tree_view_append_column(GTK_TREE_VIEW(dv), dv->column);
@@ -966,21 +971,22 @@ fb_dirview_row_activated (GtkTreeView *tview,
 			  GtkTreePath *tpath,
 			  GtkTreeViewColumn *column)
 {
-	gint type;
-	gchar *path;
+	gint type = -1;
+	gchar *path = NULL;
 	GtkTreeIter iter;
 	GtkTreeModel *model;
 	FbDirview *dv = FB_DIRVIEW (tview);
 
 	model = gtk_tree_view_get_model (tview);
-	gtk_tree_model_get_iter (model, &iter, tpath);
-	gtk_tree_model_get (model, &iter,
-			    COLUMN_PATH, &path,
-			    COLUMN_TYPE, &type,
-			    -1);
+	if (gtk_tree_model_get_iter (model, &iter, tpath)) {
+		gtk_tree_model_get (model, &iter,
+				    COLUMN_PATH, &path,
+				    COLUMN_TYPE, &type,
+				    -1);
 
-	g_signal_emit (dv, fb_dirview_signals[PATH_ACTIVATED],
-		       0, path, type);
+		g_signal_emit (dv, fb_dirview_signals[PATH_ACTIVATED],
+			       0, path, type);
+	}
 }
 
 static void
@@ -1004,7 +1010,7 @@ fb_dirview_drag_begin (GtkWidget *widget,
 
 	gtk_tree_view_get_cursor (GTK_TREE_VIEW (widget), &tpath, NULL);
 	if (gtk_tree_model_get_iter (GTK_TREE_MODEL (dv->store), &iter, tpath)) {
-		gchar *icon_id;
+		gchar *icon_id = NULL;
 
 		gtk_tree_model_get (GTK_TREE_MODEL (dv->store), &iter,
 				    COLUMN_ICON, &icon_id,
@@ -1056,10 +1062,10 @@ fb_dirview_drag_leave (GtkWidget *widget,
 	FbDirview *dv = FB_DIRVIEW (widget);
 
 	if (dv->prev_path) {
-		gtk_tree_model_get_iter (GTK_TREE_MODEL (dv->store), &iter, dv->prev_path);
-		gtk_list_store_set (dv->store, &iter,
-				    COLUMN_ICON, "fb-dir",
-				    -1);
+		if (gtk_tree_model_get_iter (GTK_TREE_MODEL (dv->store), &iter, dv->prev_path))
+			gtk_list_store_set (dv->store, &iter,
+					    COLUMN_ICON, "fb-dir",
+					    -1);
 		gtk_tree_path_free (dv->prev_path);
 		dv->prev_path = NULL;
 	}
@@ -1083,11 +1089,11 @@ fb_dirview_drag_data_received (GtkWidget *widget,
 
 /* 	g_print ("dirview:drag data received\n"); */
 
-	if (dv->prev_path && 
-	    gtk_tree_model_get_iter (GTK_TREE_MODEL (dv->store), &iter, dv->prev_path)) {
-		gtk_list_store_set (dv->store, &iter,
-				    COLUMN_ICON, "fb-dir",
-				    -1);
+	if (dv->prev_path) {
+		if (gtk_tree_model_get_iter (GTK_TREE_MODEL (dv->store), &iter, dv->prev_path))
+			gtk_list_store_set (dv->store, &iter,
+					    COLUMN_ICON, "fb-dir",
+					    -1);
 		gtk_tree_path_free (dv->prev_path);
 		dv->prev_path = NULL;
 	}
@@ -1113,23 +1119,25 @@ fb_dirview_drag_data_received (GtkWidget *widget,
 		base = g_path_get_basename (src);
 	
 		type = -1;
+		sub = NULL;
 		gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (widget), x, y, &tpath, NULL, NULL, NULL);
 		if (tpath) {
-			gtk_tree_model_get_iter (GTK_TREE_MODEL (dv->store), &iter, tpath);
-			gtk_tree_model_get (GTK_TREE_MODEL (dv->store), &iter,
-					    COLUMN_TYPE, &type,
-					    COLUMN_PATH, &sub,
-					    -1);
+			if (gtk_tree_model_get_iter (GTK_TREE_MODEL (dv->store), &iter, tpath))
+				gtk_tree_model_get (GTK_TREE_MODEL (dv->store), &iter,
+						    COLUMN_TYPE, &type,
+						    COLUMN_PATH, &sub,
+						    -1);
 			gtk_tree_path_free (tpath);
 		}
 
-		if (type = DIRECTORY && x <= ICON_WIDTH && sub)
+		if ((type == DIRECTORY || type == WORKDIR || type == PARENTDIR)
+		    && x <= ICON_WIDTH && sub)
 			dest = g_strconcat (sub, "/", base, NULL);
 		else
 			dest = g_strconcat (dv->workdir, "/", base, NULL);
 
 		g_free (base);
-		g_free (sub);
+		if (sub) g_free (sub);
 
 		if (! g_file_test (src, G_FILE_TEST_EXISTS) ||
 		    g_file_test (dest, G_FILE_TEST_EXISTS))
@@ -1186,19 +1194,20 @@ fb_dirview_drag_motion (GtkWidget *widget,
 
         if (dv->prev_path &&
 	    (x > ICON_WIDTH || ! tpath || gtk_tree_path_compare (tpath, dv->prev_path))) {
-		gtk_tree_model_get_iter (model, &iter, dv->prev_path);
-		gtk_list_store_set (dv->store, &iter,
-				    COLUMN_ICON, "fb-dir",
-				    -1);
+		if (gtk_tree_model_get_iter (model, &iter, dv->prev_path))
+			gtk_list_store_set (dv->store, &iter,
+					    COLUMN_ICON, "fb-dir",
+					    -1);
 		gtk_tree_path_free (dv->prev_path);
 		dv->prev_path = NULL;
 	}
 
 	if (tpath && x <= ICON_WIDTH) {
-		gtk_tree_model_get_iter (model, &iter, tpath);
-		gtk_tree_model_get (model, &iter,
-				    COLUMN_ICON, &icon_id,
-				    -1);
+		icon_id = NULL;
+		if (gtk_tree_model_get_iter (model, &iter, tpath))
+			gtk_tree_model_get (model, &iter,
+					    COLUMN_ICON, &icon_id,
+					    -1);
 		if (icon_id) {
 			if (! g_ascii_strcasecmp (icon_id, "fb-dir")) {
 				gtk_list_store_set (dv->store, &iter,
@@ -1225,8 +1234,8 @@ GList *
 fb_dirview_get_path_list (FbDirview *dv,
 			  FbFileType type)
 {
-	gint src_type;
-	gchar *path, *src_path;
+	gint cmp;
+	gchar *path;
 	GList *path_list = NULL;
 	GtkTreeIter iter;
 
@@ -1234,14 +1243,14 @@ fb_dirview_get_path_list (FbDirview *dv,
 
 	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(dv->store), &iter);
 	while (gtk_tree_model_iter_next(GTK_TREE_MODEL(dv->store), &iter)) {
+		cmp = -1;
+		path = NULL;
 		gtk_tree_model_get(GTK_TREE_MODEL(dv->store), &iter,
-				   COLUMN_TYPE, &src_type,
-				   COLUMN_PATH, &src_path,
+				   COLUMN_TYPE, &cmp,
+				   COLUMN_PATH, &path,
 				   -1);
-		if (src_type == type) {
-			path = g_strdup(src_path);
+		if (cmp == type)
 			path_list = g_list_append(path_list, path);
-		}
 	}
 
 	return path_list;
@@ -1327,23 +1336,24 @@ set_dir_entries (gpointer data)
 
 	do {
 		while (gtk_events_pending ()) gtk_main_iteration ();	
-
+		type = -1;
 		gtk_tree_model_get (model, &iter,
 				    COLUMN_TYPE, &type,
 				    -1);
-		if (type != DIRECTORY) continue;
+		if (type < WORKDIR || type > DIRECTORY) continue;
 
+		path = NULL;
 		gtk_tree_model_get (model, &iter,
 				    COLUMN_PATH, &path,
 				    -1);
-
-		size = str_entries2str(count_dir_entries(path, dv->dot_filter, dv->other_filter));
-		gtk_list_store_set (dv->store, &iter,
-				    COLUMN_DISPLAY_SIZE, size,
-				    -1);
-
-		g_free (path);
-		g_free (size);
+		if (path) {
+			size = str_entries2str(count_dir_entries(path, dv->dot_filter, dv->other_filter));
+			gtk_list_store_set (dv->store, &iter,
+					    COLUMN_DISPLAY_SIZE, size,
+					    -1);
+			g_free (path);
+			g_free (size);
+		}
 	} while (gtk_tree_model_iter_next (model, &iter));
 
 	dv->source_id = -1;
@@ -1378,7 +1388,7 @@ fb_dirview_real_set_dir (FbDirview *dv,
 
 	path_list = get_path_list (abs_path, FALSE);
 	for (s = path_list; s != NULL; s = s->next) {
-		dirlist_append (dv->store, s->data, dv->dot_filter, dv->other_filter);
+		dirlist_append (dv->store, s->data, dv->dot_filter, dv->other_filter, FALSE);
 	}
 	path_list_free (path_list);
 
@@ -1466,6 +1476,7 @@ fb_dirview_add_path (FbDirview *dv,
 
 	gtk_tree_model_get_iter_first (model, &iter);
 	do {
+		store = NULL;
 		gtk_tree_model_get (model, &iter,
 				    COLUMN_PATH, &store,
 				    -1);
@@ -1476,12 +1487,18 @@ fb_dirview_add_path (FbDirview *dv,
 		}
 	} while (gtk_tree_model_iter_next (model, &iter));
 
-	iter = dirlist_append (dv->store, path, dv->dot_filter, dv->other_filter);
+	iter = dirlist_append (dv->store, path, dv->dot_filter, dv->other_filter, TRUE);
 
+	type = -1;
 	gtk_tree_model_get (model, &iter,
 			    COLUMN_TYPE, &type,
 			    -1);
-	if (type == IMAGE) dv->image_num++;
+	if (type == IMAGE) {
+		dv->image_num++;
+	}
+/* 	else if (type == DIRECTORY) { */
+
+/* 	} */
 
 	tpath = gtk_tree_model_get_path (model, &iter);
 	gtk_tree_view_set_cursor (GTK_TREE_VIEW (dv), tpath, NULL, FALSE);
