@@ -1011,10 +1011,14 @@ fb_image_set_file (FbImage *image,
 
 	if (tile->path) g_free (tile->path);
 
-	tile->path = g_strdup (path);
-/* 	gdk_pixbuf_get_file_info (tile->path, &tile->raw_image_w, &tile->raw_image_h); */
-
-	fb_tile_redraw (tile, GTK_WIDGET (image));
+	if (path) {
+		tile->path = g_strdup (path);
+		fb_tile_redraw (tile, GTK_WIDGET (image));
+	}
+	else {
+		tile->path = NULL;
+		fb_tile_free_image (tile);
+	}
 }
 
 gint
@@ -1035,6 +1039,20 @@ fb_image_set_file_at_pointer (FbImage *image,
 	return g_list_index (image->tiles, tile);
 }
 
+gchar *
+fb_image_get_file_at_pointer (FbImage *image)
+{
+	FbTile *tile;
+
+	g_return_val_if_fail (FB_IS_IMAGE (image), NULL);
+
+	tile = fb_image_get_tile_at_pointer (image);
+
+	if (tile) return g_strdup (tile->path);
+
+	return NULL;
+}
+
 void
 fb_image_zoom (FbImage *image,
 	       gfloat ratio)
@@ -1290,3 +1308,49 @@ fb_image_mirror (FbImage *image,
 	tile->mirroring = ! tile->mirroring;
 	fb_tile_redraw (tile, GTK_WIDGET (image));
 }
+
+void
+fb_image_get_first_image_pos (FbImage *image,
+			      gint *x,
+			      gint *y)
+{
+	FbTile *tile;
+
+	g_return_if_fail (FB_IS_IMAGE (image));
+
+	tile = g_list_nth_data (image->tiles, 0);
+	if (tile) {
+		*x = tile->area->x;
+		*y = tile->area->y;
+	}
+	else {
+		*x = 0;
+		*y = 0;
+	}
+}
+
+void
+fb_image_get_last_image_pos (FbImage *image,
+			     gint *x,
+			     gint *y)
+{
+	gint i;
+	FbTile *tile = NULL;
+
+	g_return_if_fail (FB_IS_IMAGE (image));
+
+	for (i = g_list_length (image->tiles) - 1; i >= 0; i--) {
+		tile = g_list_nth_data (image->tiles, i);
+		if (tile && tile->path) break;
+	}
+
+	if (tile) {
+		*x = tile->area->x;
+		*y = tile->area->y;
+	}
+	else {
+		*x = 0;
+		*y = 0;
+	}
+}
+
