@@ -77,6 +77,9 @@ static void     cb_drag_data_received (GtkWidget *icon_view,
 				       guint info,
 				       guint time,
 				       gpointer data);
+static void     cb_item_activated     (GtkIconView *icon_view,
+				       GtkTreePath *arg1,
+				       gpointer data);
 static gboolean cb_motion_notify      (GtkWidget *icon_view,
 				       GdkEventMotion *event,
 				       gpointer data);
@@ -259,7 +262,7 @@ fb_thumbwin_init (FbThumbWin *tw)
 
 	tw->ui_manager = fb_thumbwin_ui_manager_get (tw);
 
-	gtk_window_set_title(GTK_WINDOW(tw), "Thumbnail - Futaba");
+	gtk_window_set_title(GTK_WINDOW(tw), "- Futaba Thumbnail -");
 	gtk_window_add_accel_group(GTK_WINDOW(tw), gtk_ui_manager_get_accel_group(tw->ui_manager));
 
 	tw->popup1 = gtk_ui_manager_get_widget (tw->ui_manager, "/OverThumbPopup");
@@ -275,10 +278,10 @@ fb_thumbwin_init (FbThumbWin *tw)
 	gtk_widget_show (tw->scroll);
 
 	tw->store = gtk_list_store_new (THUMBWIN_COLUMNS,
-					      GDK_TYPE_PIXBUF,
-					      G_TYPE_STRING,
-					      G_TYPE_STRING,
-					      -1);
+					GDK_TYPE_PIXBUF,
+					G_TYPE_STRING,
+					G_TYPE_STRING,
+					-1);
 
 	sortable = GTK_TREE_SORTABLE (tw->store);
 	gtk_tree_sortable_set_sort_func(sortable,
@@ -302,6 +305,7 @@ fb_thumbwin_init (FbThumbWin *tw)
 	gtk_icon_view_set_row_spacing (GTK_ICON_VIEW (tw->icon_view), 10);
 	gtk_icon_view_set_column_spacing (GTK_ICON_VIEW (tw->icon_view), 10);
 	gtk_icon_view_set_margin (GTK_ICON_VIEW (tw->icon_view), THUMBWIN_MARGIN);
+	gtk_icon_view_set_selection_mode (GTK_ICON_VIEW (tw->icon_view), GTK_SELECTION_MULTIPLE);
 	gtk_drag_dest_set(GTK_WIDGET(tw->icon_view), 
 			  GTK_DEST_DEFAULT_HIGHLIGHT |
 			  GTK_DEST_DEFAULT_MOTION | 
@@ -316,6 +320,8 @@ fb_thumbwin_init (FbThumbWin *tw)
 			  G_CALLBACK (cb_drag_data_get), tw);
 	g_signal_connect (G_OBJECT (tw->icon_view), "drag-data-received",
 			  G_CALLBACK (cb_drag_data_received), tw);
+	g_signal_connect (G_OBJECT (tw->icon_view), "item-activated",
+			  G_CALLBACK (cb_item_activated), tw);
 	g_signal_connect (G_OBJECT (tw->icon_view), "motion-notify-event",
 			  G_CALLBACK (cb_motion_notify), tw);
 	gtk_container_add (GTK_CONTAINER (tw->scroll), tw->icon_view);
@@ -431,16 +437,23 @@ cb_button_press (GtkWidget *icon_view,
 	}
 		break;
 	case 3: {
+		gtk_icon_view_unselect_all (GTK_ICON_VIEW (icon_view));
+		gtk_tree_path_free (tw->prev_path);
+		tw->prev_path = NULL;
+
+		tw->prev_path = gtk_icon_view_get_path_at_pos (GTK_ICON_VIEW (icon_view), event->x, event->y);
 		if (tw->prev_path) {
+			gtk_icon_view_select_path (GTK_ICON_VIEW (icon_view), tw->prev_path);
 			gtk_menu_popup(GTK_MENU(tw->popup1), NULL, NULL, NULL, NULL,
 				       event->button, event->time);
+
 		}
 		else {
 			gboolean flag;
 			GtkWidget *widget;
 
 			widget = gtk_ui_manager_get_widget (tw->ui_manager, "/Popup/Paste");
-			flag = (paste_path != NULL);
+			flag = (paste_list != NULL);
 			gtk_widget_set_sensitive (widget, flag);
 
 			gtk_menu_popup(GTK_MENU(tw->popup2), NULL, NULL, NULL, NULL,
@@ -464,22 +477,27 @@ cb_drag_begin (GtkWidget *icon_view,
 	FbThumbWin *tw = data;
 
 /* 	g_print ("icon_view:drag begin\n"); */
+        /* FIXME can multiple items dnd */
+	gtk_icon_view_unselect_all (GTK_ICON_VIEW (icon_view));
 
 	if (tw->prev_path && gtk_tree_model_get_iter (GTK_TREE_MODEL (tw->store), &iter, tw->prev_path)) {
-		gchar *path;
-		GdkPixbuf *pixbuf;
+		gchar *path = NULL;
+		GdkPixbuf *pixbuf = NULL;
 
 		gtk_tree_model_get (GTK_TREE_MODEL (tw->store), &iter,
 				    THUMBWIN_PATH, &path,
 				    THUMBWIN_PIXBUF, &pixbuf,
 				    -1);
 
-		if (tw->drag_data) g_free (tw->drag_data);
-		tw->drag_data = g_strdup_printf ("file://%s\r\n", path);
-		g_free (path);
-
-		gtk_drag_set_icon_pixbuf (drag_context, pixbuf, -5, -5);
-		g_object_unref (pixbuf);
+		if (path) {
+			if (tw->drag_data) g_free (tw->drag_data);
+			tw->drag_data = g_strdup_printf ("file://%s\r\n", path);
+			g_free (path);
+		}
+		if (pixbuf) {
+			gtk_drag_set_icon_pixbuf (drag_context, pixbuf, -5, -5);
+			g_object_unref (pixbuf);
+		}
 	}
 }
 
@@ -579,6 +597,26 @@ cb_drag_data_received (GtkWidget *icon_view,
 	g_free (uri);
 }
 
+static void
+cb_item_activated (GtkIconView *icon_view,
+		   GtkTreePath *arg1,
+		   gpointer data)
+{
+	gint x, y;
+	FbThumbWin *tw = FB_THUMBWIN (data);
+
+	gtk_widget_get_pointer (GTK_WIDGET (icon_view), &x, &y);
+	if (gtk_icon_view_over_pixbuf (icon_view, x, y)) {
+		GtkAction *action;
+
+		action = gtk_ui_manager_get_action (tw->ui_manager, "/OverThumbPopup/OpenViewer");
+		if (action) g_signal_emit_by_name (G_OBJECT (action), "activate");
+	}
+	else {
+		/* rename */
+	}
+}
+
 static gboolean
 cb_motion_notify(GtkWidget *icon_view,
 		 GdkEventMotion *event,
@@ -595,7 +633,7 @@ cb_motion_notify(GtkWidget *icon_view,
 				    GDK_ACTION_ASK  | GDK_ACTION_COPY
 				    | GDK_ACTION_MOVE | GDK_ACTION_LINK);
 	}
-	else if (! tpath && tw->prev_path && event->type != GDK_DRAG_MOTION) {
+	else if (! tpath && tw->prev_path) {
 		gtk_drag_source_unset (icon_view);
 	}
 
@@ -651,6 +689,8 @@ fb_thumbwin_start (GList *path_list)
 
 		fb_thumbwin_add_thumbnail (FB_THUMBWIN (tw), p->data);
 	}
+
+	fb_thumbwin_update_title (FB_THUMBWIN (tw));
 }
 
 void
@@ -687,3 +727,61 @@ fb_thumbwin_add_thumbnail (FbThumbWin *tw,
 	g_free (text);
 	g_free (utf8);
 }
+
+void
+fb_thumbwin_update_thumbnail (FbThumbWin *tw)
+{
+	gchar *path;
+	GtkTreeIter iter;
+	GtkTreeModel *model;
+
+	g_return_if_fail (FB_IS_THUMBWIN (tw));
+
+	model = GTK_TREE_MODEL (tw->store);
+	gtk_tree_model_get_iter_first (model, &iter);
+	do {
+		path = NULL;
+		gtk_tree_model_get (model, &iter,
+				    THUMBWIN_PATH, &path,
+				    -1);
+		if (path) {
+			if (! path_is_exists (path))
+				gtk_list_store_remove (tw->store, &iter);
+			g_free (path);
+		}
+	} while (gtk_tree_model_iter_next (model, &iter));
+
+	fb_thumbwin_update_title (tw);
+}
+
+void
+fb_thumbwin_update_title (FbThumbWin *tw)
+{
+	gint n_items;
+	gchar *title;
+
+	g_return_if_fail (FB_IS_THUMBWIN (tw));
+
+	n_items = gtk_icon_view_get_n_items (GTK_ICON_VIEW (tw->icon_view));
+	if (n_items == 0)
+		title = g_strdup_printf ("\"%s\" - Futaba Thumbnail -", 
+					 tw->workdir);
+	else if (n_items == 1)
+		title = g_strdup_printf ("%d item - Futaba Thumbnail -", 
+					 n_items);
+	else if (n_items > 1)
+		title = g_strdup_printf ("%d items - Futaba Thumbnail -", 
+					 n_items);
+
+	gtk_window_set_title(GTK_WINDOW(tw), title);
+	g_free (title);
+}
+
+GList *
+fb_thumbwin_get_selected_items (FbThumbWin *tw)
+{
+	g_return_val_if_fail (FB_IS_THUMBWIN (tw), NULL);
+
+	return gtk_icon_view_get_selected_items (GTK_ICON_VIEW (tw->icon_view));
+}
+
