@@ -39,7 +39,7 @@
 #include "fb-viewer.h"
 #include "fr-archive.h"
 
-gchar *paste_path = NULL;
+GList *paste_list = NULL;
 gboolean is_copy = TRUE;
 /* futaba window */
 static void     fb_window_class_init     (FbWindowClass *klass);
@@ -478,7 +478,7 @@ cb_show_popup (GtkWidget *wg,
 	ui_set_sensitive(fb->ui_manager, "/DirViewPopup/Rename", flag);
 	/*       ui_set_sensitive(fb->ui_manager, "/DirViewPopup/ChangeBulk", flag); */
 	ui_set_sensitive(fb->ui_manager, "/DirViewPopup/MakeDir", flag);
-	flag = (flag && (paste_path != NULL));
+	flag = (flag && (paste_list != NULL));
 	ui_set_sensitive(fb->ui_manager, "/DirViewPopup/Paste", flag);
 
 
@@ -546,14 +546,14 @@ cb_change_preview (GtkTreeView *tview,
 	else
 		set_path = g_strdup (path);
 
-	if (type == DIRECTORY)
+	if (type <= DIRECTORY)
 		fb_statusbar_push(_("\"%s\" selected (containing %s)"), set_path, size);
 	else
 		fb_statusbar_push(_("\"%s\" selected (size %s)"), set_path, size);
 	g_free (set_path);
 	g_free (size);
 
-	if ((type == DIRECTORY) || (type == ARCHIVE) || (type == OTHER)) 
+	if (type != IMAGE)
 		goto fin;
 
 	if (! GTK_WIDGET_VISIBLE (fb->preview)) return;
@@ -601,6 +601,8 @@ cb_set_path (FbDirview *dv,
 	}
 
 	switch (type) {
+	case WORKDIR:
+	case PARENTDIR:
 	case DIRECTORY: {
 		fb_dirview_set_dir (dv, path);
 	}
