@@ -61,6 +61,23 @@ path_is_file (const gchar *path)
 	return S_ISREG(buf.st_mode);
 }
 
+gboolean
+path_is_image (const gchar *path)
+{
+	if (! path || ! *path) return FALSE;
+	if (! path_is_file (path)) return FALSE;
+
+	return file_is_image (path);
+}
+
+gboolean
+path_is_archive (const gchar *path)
+{
+	if (! path || ! *path) return FALSE;
+	if (! path_is_file (path)) return FALSE;
+
+	return file_is_archive (path);
+}
 
 gboolean
 path_is_dir (const gchar *path)
@@ -781,7 +798,7 @@ cpdir_recursive (const gchar *from,
 		char *src = scan->data;
 		char *dest;
 
-		dest = g_strconcat (to, "/", file_name_from_path (src));
+		dest = g_strconcat (to, "/", file_name_from_path (src), NULL);
 		if (! ensure_dir_exists (dest, 0755))
 			error = TRUE;
 		g_free (dest);
@@ -830,12 +847,12 @@ rmdir_recursive (const gchar *directory)
 		if (rmdir_recursive (sub_dir) == FALSE)
 			error = TRUE;
 		if (rmdir (sub_dir) == 0)
-			error = TRUE;
+			error = FALSE;
 	}
 	path_list_free (dirs);
 
 	if (rmdir (directory) == 0)
-		error = TRUE;
+		error = FALSE;
 
 	return !error;
 }
@@ -844,7 +861,7 @@ gboolean
 mvdir_recursive (const gchar *from,
 		 const gchar *to)
 {
-	if (cpdir_recursive (from, to) && ! rmdir_recursive (from))
+	if (cpdir_recursive (from, to) && rmdir_recursive (from))
 		return TRUE;
 
 	return FALSE;
